from pathlib import Path
import subprocess
from typing import Optional

from loguru import logger
from phylorun.engines.engine import Engine


class RevBayes(Engine):
    def name(self) -> str:
        """Returns the name of the engine as used in the CLI."""
        return "revbayes"

    def can_run_analysis(self, analysis_file: Path) -> bool:
        """Checks if RevBayes can run the analysis in the given file."""
        if not analysis_file.name.endswith(".rev"):
            logger.debug("No Rev file: wrong file extension (.rev requried).")
            return False

        logger.debug("RevBayes file found.")
        return True

    def run_local_analysis(
        self,
        analysis_file: Path,
        engine_path: Optional[str] = None,
        additional_cli_args: Optional[list[str]] = None,
    ):
        """Runs the analysis in the given file using the locally installed engine."""
        engine_path = engine_path or "rb"
        if not engine_path:
            raise Exception("No RevBayes binary found.")

        additional_cli_args = additional_cli_args or []

        subprocess.run([engine_path, *additional_cli_args, analysis_file])

    def run_containerized_analysis(
        self, analysis_file: Path, additional_cli_args: Optional[list[str]] = None
    ):
        """Runs the analysis in the given file in a container. This does not require the
        engine to be installed on the system."""
        logger.info("Run container RevBayes")
        raise NotImplementedError



import io
import sys
import docker
from docker.models.containers import Container
from docker.errors import DockerException, ImageNotFound
from loguru import logger


def get_docker_client() -> docker.DockerClient:
    try:
        return docker.from_env()
    except DockerException:
        logger.error("Docker could not be instantiated. Is it installed and running?")
        raise Exception(
            "Docker could not be instantiated. Is it installed and running?"
        )


def create_image_if_needed(
    client: docker.DockerClient, image_name: str, docker_file: str
):
    # try:
    #     client.images.get(image_name)
    #     # image already created
    #     return
    # except ImageNotFound:
    #     # we still need to create the image
    #     ...

    docker_file_bytes = io.BytesIO(docker_file.encode("utf-8"))

    client.images.build(
        fileobj=docker_file_bytes,
        tag=image_name,
        rm=True,
        platform="linux/x86_64",
    )


def start_container(
    client: docker.DockerClient, image_name: str, **kwargs
) -> Container:
    return client.containers.run(
        image_name, "sleep infinity", detach=True, platform="linux/x86_64", **kwargs
    )


def run_and_print_command(container: Container, command: str):
    result = container.exec_run(command, stream=True, demux=True)

    for stdout, stderr in result.output:
        if stdout:
            print(stdout.decode(), end="")
        if stderr:
            print(stderr.decode(), end="", file=sys.stderr)
